package javax.xml.crypto.dsig;

import javax.xml.crypto.URIDereferencer;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.XMLStructure;
import javax.xml.crypto.dsig.spec.TransformParameterSpec;
import javax.xml.crypto.dsig.spec.SignatureMethodParameterSpec;
import java.security.InvalidAlgorithmParameterException;
import javax.xml.crypto.dsig.spec.DigestMethodParameterSpec;
import javax.xml.crypto.Data;
import java.util.List;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import javax.xml.crypto.NoSuchMechanismException;
import sun.security.jca.GetInstance;
import java.security.Provider;

public abstract class XMLSignatureFactory
{
    private String mechanismType;
    private Provider provider;
    
    protected XMLSignatureFactory() {
    }
    
    public static XMLSignatureFactory getInstance(String s) {
        if (s == null) {
            throw new NullPointerException("mechanismType cannot be null");
        }
        GetInstance.Instance instance;
        try {
            instance = GetInstance.getInstance("XMLSignatureFactory", null, s);
        }
        catch (NoSuchAlgorithmException cause) {
            throw new NoSuchMechanismException(cause);
        }
        XMLSignatureFactory xmlSignatureFactory = (XMLSignatureFactory)instance.impl;
        xmlSignatureFactory.mechanismType = s;
        xmlSignatureFactory.provider = instance.provider;
        return xmlSignatureFactory;
    }
    
    public static XMLSignatureFactory getInstance(String s, Provider provider) {
        if (s == null) {
            throw new NullPointerException("mechanismType cannot be null");
        }
        if (provider == null) {
            throw new NullPointerException("provider cannot be null");
        }
        GetInstance.Instance instance;
        try {
            instance = GetInstance.getInstance("XMLSignatureFactory", null, s, provider);
        }
        catch (NoSuchAlgorithmException cause) {
            throw new NoSuchMechanismException(cause);
        }
        XMLSignatureFactory xmlSignatureFactory = (XMLSignatureFactory)instance.impl;
        xmlSignatureFactory.mechanismType = s;
        xmlSignatureFactory.provider = instance.provider;
        return xmlSignatureFactory;
    }
    
    public static XMLSignatureFactory getInstance(String s, String provider) throws NoSuchProviderException {
        if (s == null) {
            throw new NullPointerException("mechanismType cannot be null");
        }
        if (provider == null) {
            throw new NullPointerException("provider cannot be null");
        }
        if (provider.length() == 0) {
            throw new NoSuchProviderException();
        }
        GetInstance.Instance instance;
        try {
            instance = GetInstance.getInstance("XMLSignatureFactory", null, s, provider);
        }
        catch (NoSuchAlgorithmException cause) {
            throw new NoSuchMechanismException(cause);
        }
        XMLSignatureFactory xmlSignatureFactory = (XMLSignatureFactory)instance.impl;
        xmlSignatureFactory.mechanismType = s;
        xmlSignatureFactory.provider = instance.provider;
        return xmlSignatureFactory;
    }
    
    public static XMLSignatureFactory getInstance() {
        return getInstance("DOM");
    }
    
    public final String getMechanismType() {
        return this.mechanismType;
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public abstract XMLSignature newXMLSignature(SignedInfo p0, KeyInfo p1);
    
    public abstract XMLSignature newXMLSignature(SignedInfo p0, KeyInfo p1, List p2, String p3, String p4);
    
    public abstract Reference newReference(String p0, DigestMethod p1);
    
    public abstract Reference newReference(String p0, DigestMethod p1, List p2, String p3, String p4);
    
    public abstract Reference newReference(String p0, DigestMethod p1, List p2, String p3, String p4, byte[] p5);
    
    public abstract Reference newReference(String p0, DigestMethod p1, List p2, Data p3, List p4, String p5, String p6);
    
    public abstract SignedInfo newSignedInfo(CanonicalizationMethod p0, SignatureMethod p1, List p2);
    
    public abstract SignedInfo newSignedInfo(CanonicalizationMethod p0, SignatureMethod p1, List p2, String p3);
    
    public abstract XMLObject newXMLObject(List p0, String p1, String p2, String p3);
    
    public abstract Manifest newManifest(List p0);
    
    public abstract Manifest newManifest(List p0, String p1);
    
    public abstract SignatureProperty newSignatureProperty(List p0, String p1, String p2);
    
    public abstract SignatureProperties newSignatureProperties(List p0, String p1);
    
    public abstract DigestMethod newDigestMethod(String p0, DigestMethodParameterSpec p1) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException;
    
    public abstract SignatureMethod newSignatureMethod(String p0, SignatureMethodParameterSpec p1) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException;
    
    public abstract Transform newTransform(String p0, TransformParameterSpec p1) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException;
    
    public abstract Transform newTransform(String p0, XMLStructure p1) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException;
    
    public abstract CanonicalizationMethod newCanonicalizationMethod(String p0, C14NMethodParameterSpec p1) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException;
    
    public abstract CanonicalizationMethod newCanonicalizationMethod(String p0, XMLStructure p1) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException;
    
    public final KeyInfoFactory getKeyInfoFactory() {
        return KeyInfoFactory.getInstance(this.getMechanismType(), this.getProvider());
    }
    
    public abstract XMLSignature unmarshalXMLSignature(XMLValidateContext p0) throws MarshalException;
    
    public abstract XMLSignature unmarshalXMLSignature(XMLStructure p0) throws MarshalException;
    
    public abstract boolean isFeatureSupported(String p0);
    
    public abstract URIDereferencer getURIDereferencer();
}
